@@ -4,13 +4,13 @@
  - ltcltest.lua
  -
  - an attempt at a test suite for ltcl interpreter integration. Probably not comprehensive, but
- - should cover a most of the functionality.
+ - should cover most of the functionality.
  -
  - Gunnar Zötl <gz@tset.de>, 2010
  - Released under MIT/X11 license. See file LICENSE for details.
 --]]
 
-require "ltcl"
+ltcl = require "ltcl"
 
 tcl = ltcl.new()
 
@@ -224,14 +224,23 @@ else
 	fail(tostring(val))
 end
 
+-- 21
+announce("converting string '1 2 3 4' to table using list2table, result should be a table {1, 2, 3, 4}")
+ok, val = pcall(ltcl.list2table, tcl, '1 2 3 4')
+if ok then
+	check(cmptables(val, {1,2,3,4}))
+else
+	fail(tostring(val))
+end
+
 -- array tests
 
--- 21
+-- 22
 announce("Attempting to refer to index 1 or array variable varr1, should fail")
 ok, msg = pcall(ltcl.getarray, tcl, 'varr1', 1)
 check(not ok)
 
--- 22
+-- 23
 announce("creating an array {'a' 1 'b' 2 'c' 3} from tcl, then read indices 'a', 'b', 'c' from it. Should return 1, 2, 3 in turn.")
 ok, msg = pcall(ltcl.call, tcl, 'array', 'set', 'varr1', {'a', 1, 'b', 2, 'c', 3})
 if ok then
@@ -246,7 +255,7 @@ else
 	fail(tostring(msg))
 end
 
--- 23
+-- 24
 announce("adding 'd'=4 to varr1, then read index 'd' back, should return a number 4")
 ok, msg = pcall(ltcl.setarray, tcl, 'varr1', 'd', 4)
 if ok then
@@ -260,19 +269,19 @@ else
 	fail(tostring(msg))
 end
 
--- 24
+-- 25
 announce("retrieving undefined index 'e' from varr1, should fail")
 ok = pcall(ltcl.getarray, tcl, 'varr1', 'e')
 check(not ok)
 
--- 25
+-- 26
 announce("creating recursive table a={a}, then setting Tcl var to it. Should fail.")
 rectbl={}
 rectbl[1]=rectbl
 ok = pcall(ltcl.setvar, tcl, 'rectbl', rectbl)
 check(not ok)
 
--- 26
+-- 27
 announce("creating recursive table a={{a[2]},{a[1]}}, then setting Tcl var to it. Should fail.")
 rectbl={}
 rectbl[1]={}
@@ -288,7 +297,7 @@ check(not ok)
 
 -- tests basic tcl expression evaluation, result conversion and error propagation
 
--- 27
+-- 28
 announce("Calling makearglist with {1,2,3}, should return {1,2,3}")
 ok, val = pcall(ltcl.makearglist, tcl, {1,2,3})
 if ok then
@@ -297,7 +306,7 @@ else
 	fail(tostring(val))
 end
 
--- 28
+-- 29
 announce("TCalling 'list' with a 200 parameter array, should return a table with said 200 values")
 parms = {}
 for i=1,200 do parms[i] = i end
@@ -308,7 +317,7 @@ else
 	fail(tostring(val))
 end
 
--- 29
+-- 30
 announce("TCalling 'list' with a 100 entry parameter hash, should return a 200 entry list with matching entries")
 hparms = {}
 for i=1,100 do hparms["key"..tostring(i)] = "val"..tostring(i) end
@@ -327,7 +336,7 @@ else
 	fail(tostring(val))
 end
 
--- 30
+-- 31
 announce("Calling makearglist with {1,2,3,c=4}, should return {1,2,3, '-c',4}")
 ok, val = pcall(ltcl.makearglist, tcl, {1,2,3,c=4})
 if ok then
@@ -336,7 +345,7 @@ else
 	fail(tostring(val))
 end
 
--- 31
+-- 32
 announce("Evaluating 'return $vnum', result should be a number 1")
 ok, val = pcall(ltcl.eval, tcl, 'return $vnum')
 if ok then
@@ -345,7 +354,7 @@ else
 	fail(tostring(val))
 end
 
--- 32
+-- 33
 announce("Evaluating 'return $vbool', result should be a number 1")
 ok, val = pcall(ltcl.eval, tcl, 'return $vbool')
 if ok then
@@ -354,7 +363,7 @@ else
 	fail(tostring(val))
 end
 
--- 33
+-- 34
 announce("Evaluating 'return $vstr', result should be a string 'Hallo'")
 ok, val = pcall(ltcl.eval, tcl, 'return $vstr')
 if ok then
@@ -363,7 +372,7 @@ else
 	fail(tostring(val))
 end
 
--- 34
+-- 35
 announce("Evaluating 'return $vlst', result should be a table {1, 2, 3, 'a', 'b', 'c'}")
 ok, val = pcall(ltcl.eval, tcl, 'return $vlst')
 if ok then
@@ -372,7 +381,7 @@ else
 	fail(tostring(val))
 end
 
--- 35
+-- 36
 announce("Evaluating 'return $vnlst', result should be a table {1, {2, {3, 4}, 5}, 6}")
 ok, val = pcall(ltcl.eval, tcl, 'return $vnlst')
 if ok then
@@ -381,7 +390,7 @@ else
 	fail(tostring(val))
 end
 
--- 36
+-- 37
 announce("Evaluating 'return $vnil, result should be an empty string ''")
 ok, val = pcall(ltcl.eval, tcl, 'return $vnil')
 if ok then
@@ -390,12 +399,12 @@ else
 	fail(tostring(val))
 end
 
--- 37
+-- 38
 announce("Evaluating 'bla fasel', should fail")
 ok = pcall(ltcl.eval, tcl, 'bla fasel')
 check(not ok)
 
--- 38
+-- 39
 announce("Evaluating 'expr 1+2', result should be a number 3")
 ok, val = pcall(ltcl.eval, tcl, 'expr 1+2')
 if ok then
@@ -404,7 +413,7 @@ else
 	fail(tostring(val))
 end
 
--- 39
+-- 40
 announce("Evaluating 'return \"Hallo\"', result should be a string 'Hallo'")
 ok, val = pcall(ltcl.eval, tcl, 'return "Hallo"')
 if ok then
@@ -413,7 +422,7 @@ else
 	fail(tostring(val))
 end
 
--- 40
+-- 41
 announce("Evaluating 'list 1 2 3', result should be a table {1,2,3}")
 ok, val = pcall(ltcl.eval, tcl, 'list 1 2 3')
 if ok then
@@ -422,7 +431,7 @@ else
 	fail(tostring(val))
 end
 
--- 41
+-- 42
 announce("Evaluating 'list 1 [list 2 3] 4', result should be a table {1,{2,3},4}")
 ok, val = pcall(ltcl.eval, tcl, 'list 1 [list 2 3] 4')
 if ok then
@@ -435,7 +444,7 @@ end
 
 -- tests parameter conversion and error propagation, everything else should be reasonably well tested by now
 
--- 42
+-- 43
 announce("Calling 'expr' with parameters 1, '+', 2, result should be a number 3")
 ok, val = pcall(ltcl.call, tcl, 'expr', 1, '+', 2)
 if ok then
@@ -444,7 +453,7 @@ else
 	fail(tostring(val))
 end
 
--- 43
+-- 44
 announce("Calling 'list' with parameters 1,2,3, result should be a table {1,2,3}")
 ok, val = pcall(ltcl.call, tcl, 'list', 1, 2, 3)
 if ok then
@@ -453,7 +462,7 @@ else
 	fail(tostring(val))
 end
 
--- 44
+-- 45
 announce("Calling 'concat' with parameter 'Hallo', result should be a string 'Hallo'")
 ok, val = pcall(ltcl.call, tcl, 'concat', 'Hallo')
 if ok then
@@ -462,22 +471,22 @@ else
 	fail(tostring(val))
 end
 
--- 45
+-- 46
 announce("Calling 'bla' without parameter, should fail")
 ok = pcall(ltcl.call, tcl, 'bla')
 check(not ok)
 
--- 46
+-- 47
 announce("Calling 'bla' with parameter 'fasel', should fail")
 ok = pcall(ltcl.call, tcl, 'bla', 'fasel')
 check(not ok)
 
--- 47
+-- 48
 announce("Calling 'puts' with parameters 1,2, should fail")
 ok = pcall(ltcl.call, tcl, 'puts', 1, 2)
 check(not ok)
 
--- 48
+-- 49
 announce("TCalling 'list' with parameter array {1,2,3}, should return a table {1,2,3}")
 ok, val = pcall(ltcl.callt, tcl, 'list', {1,2,3})
 if ok then
@@ -486,7 +495,7 @@ else
 	fail(tostring(val))
 end
 
--- 49
+-- 50
 announce("Making argument list from {1,2,c=3} then TCalling list with this list, should return a table {1,2,'-c',3}")
 ok, args = pcall(ltcl.makearglist, tcl, {1,2,c=3})
 ok, val = pcall(ltcl.callt, tcl, 'list', args)
@@ -496,7 +505,7 @@ else
 	fail(tostring(val))
 end
 
--- 50
+-- 51
 announce("Making argument list from {1,c=2,3} then TCalling list with this list, should return a table {1,3,'-c',2}")
 ok, args = pcall(ltcl.makearglist, tcl, {1,c=2,3})
 ok, val = pcall(ltcl.callt, tcl, 'list', args)
@@ -506,20 +515,58 @@ else
 	fail(tostring(val))
 end
 
+-- 52
+announce("TCalling 'list' with parameter 'a', 'b', 'c', {1,2,3}, should return a table {'a','b','c',1,2,3}")
+ok, val = pcall(ltcl.callt, tcl, 'list', 'a', 'b', 'c', {1,2,3})
+if ok then
+	check(cmptables(val, {'a','b','c',1,2,3}))
+else
+	fail(tostring(val))
+end
+
+-- 53
+announce("Making argument list from {1,2,c=3} then TCalling list with 'a', 'b', this list, should return a table {'a','b',1,2,'-c',3}")
+ok, args = pcall(ltcl.makearglist, tcl, {1,2,c=3})
+ok, val = pcall(ltcl.callt, tcl, 'list', 'a', 'b', args)
+if ok then
+	check(cmptables(val, {'a','b',1,2,'-c',3}))
+else
+	fail(tostring(val))
+end
+
+-- 54
+announce("TCalling 'list' with arguments 1, 2, 3, should return table {1,2,3}")
+ok, val = pcall(ltcl.callt, tcl, 'list', 1,2,3)
+if ok then
+	check(cmptables(val, {1,2,3}))
+else
+	fail(tostring(val))
+end
+
+-- 55
+announce("PTCalling 'list' with argument {1, 2, 3}, should fail")
+ok, val = pcall(ltcl.callpt, tcl, 'list', {1,2,3})
+check(not ok)
+
+-- 56
+announce("PTCalling 'list' with argument {1, 2, 3},4,  should fail")
+ok, val = pcall(ltcl.callpt, tcl, 'list', {1,2,3}, 4)
+check(not ok)
+
 -- lua functions from tcl 1
 
 -- tests registering lua functions with the interpreter, calling them, parameter, return values and error propagation
 
--- 51
+-- 57
 announce("Registering a simple function with tcl, taking no arguments and returning nothing")
 function lt1() end
 check(pcall(ltcl.register, tcl, 'lt1', lt1))
 
--- 52
+-- 58
 announce("Checking wether function has been registered")
 check(type(getmetatable(tcl)['__functions']['lt1']) == 'function')
 
--- 53
+-- 59
 announce("Calling said function through ltcl.eval, should return an empty string")
 ok, val = pcall(ltcl.eval, tcl, 'lt1')
 if ok then
@@ -528,7 +575,7 @@ else
 	fail(tostring(val))
 end
 
--- 54
+-- 60
 announce("Calling said function through ltcl.call, should return an empty string")
 ok, val = pcall(ltcl.call, tcl, 'lt1')
 if ok then
@@ -537,12 +584,12 @@ else
 	fail(tostring(val))
 end
 
--- 55
+-- 61
 announce("Registering a function with tcl returning its first argument")
 function lt2(x) return x end
 check(pcall(ltcl.register, tcl, 'lt2', lt2))
 
--- 56
+-- 62
 announce("Calling said function through eval with argument 1, should return 1")
 ok, val = pcall(ltcl.eval, tcl, 'lt2 1')
 if ok then
@@ -551,7 +598,7 @@ else
 	fail(tostring(val))
 end
 
--- 57
+-- 63
 announce("Calling said function through eval with argument [expr 1], should return number 1")
 ok, val = pcall(ltcl.eval, tcl, 'lt2 [expr 1]')
 if ok then
@@ -560,7 +607,7 @@ else
 	fail(tostring(val))
 end
 
--- 58
+-- 64
 announce("Calling said function through eval with argument 'Hallo', should return string 'Hallo'")
 ok, val = pcall(ltcl.eval, tcl, 'lt2 Hallo')
 if ok then
@@ -569,12 +616,12 @@ else
 	fail(tostring(val))
 end
 
--- 59
+-- 65
 announce("Registering a function with tcl returning a list of its arguments")
 function lt3(...) return {...} end
 check(pcall(ltcl.register, tcl, 'lt3', lt3))
 
--- 60
+-- 66
 announce("Calling said function through ltcl.eval with arguments 1,2,3, should return table {1,2,3}")
 ok, val = pcall(ltcl.eval, tcl, 'lt3 1 2 3')
 if ok then
@@ -583,7 +630,7 @@ else
 	fail(tostring(val))
 end
 
--- 61
+-- 67
 announce("Calling said function through ltcl.call with arguments 4,5,6, should return table {4,5,6}")
 ok, val = pcall(ltcl.call, tcl, 'lt3', 4, 5, 6)
 if ok then
@@ -592,7 +639,7 @@ else
 	fail(tostring(val))
 end
 
--- 62
+-- 68
 announce("Registering a function then calling it then unregistering it")
 function lt4() return "X" end
 ok = pcall(ltcl.register, tcl, 'lt4', lt4)
@@ -600,11 +647,11 @@ ok1, val = pcall(ltcl.call, tcl, 'lt4')
 ok = ok and ok1 and val == "X"
 check(ok and pcall(ltcl.unregister, tcl, 'lt4'))
 
--- 63
+-- 69
 announce("Checking wether unregistered function has indeed been removed")
 check(getmetatable(tcl)['__functions']['lt4'] == nil)
 
--- 64
+-- 70
 announce("Trying to call unregistered function, should fail")
 ok = pcall(ltcl.eval, tcl, 'lt4')
 ok = ok or pcall(ltcl.call, tcl, 'lt4')
@@ -614,7 +661,7 @@ check(not ok)
 
 -- tests direct calling of lua functions from tcl.
 
--- 65
+-- 71
 announce("Calling a lua function that compares its argument to number 1 with argument 1, should return 1")
 function dt1(arg) return arg == 1 or arg == '1' end
 ok, val = pcall(ltcl.eval, tcl, 'lua dt1 1')
@@ -624,7 +671,7 @@ else
 	fail(tostring(val))
 end
 
--- 66
+-- 72
 announce("Calling a lua function that compares its argument to string 'Hallo' with argument 'Hallo', should return 1")
 function dt2(arg) return arg == 'Hallo' end
 ok, val = pcall(ltcl.eval, tcl, 'lua dt2 Hallo')
@@ -634,7 +681,7 @@ else
 	fail(tostring(val))
 end
 
--- 67
+-- 73
 announce("Calling said function with a string 'Huhu', should return 0")
 ok, val = pcall(ltcl.eval, tcl, 'lua dt2 Huhu')
 if ok then
@@ -643,7 +690,7 @@ else
 	fail(tostring(val))
 end
 
--- 68
+-- 74
 announce("Calling a lua function that compares its argument to table {1,2,3} with argument [list 1 2 3], should return 1")
 function dt3(arg) return cmptables(arg, {1,2,3}) end
 ok, val = pcall(ltcl.eval, tcl, 'lua dt3 [list 1 2 3]')
@@ -653,12 +700,12 @@ else
 	fail(tostring(val))
 end
 
--- 69
+-- 75
 announce("Calling a nonexistent lua function, should fail")
 ok = pcall(ltcl.eval, tcl, 'lua dt4')
 check(not ok)
 
--- 70
+-- 76
 announce("Calling a lua function that returns all of its arguments with parameters 1 2 3, should return number 1")
 function dt4(...) return ... end
 ok, val = pcall(ltcl.eval, tcl, 'lua dt4 1 2 3')
@@ -673,7 +720,7 @@ end
 -- very basic tests, just see wether it does anything at all just using ASCII
 -- chars (which should convert to themselves), and see wether it fails when it should.
 
--- 71
+-- 77
 announce("Converting string 'abc' to utf8, result should be a string 'abc'")
 ok, val = pcall(ltcl.toutf8, tcl, 'abc')
 if ok then
@@ -682,7 +729,7 @@ else
 	fail(tostring(val))
 end
 
--- 72
+-- 78
 announce("Converting that string 'abc' from utf8, result should be a string 'abc'")
 ok, val = pcall(ltcl.fromutf8, tcl, val)
 if ok then
@@ -691,19 +738,19 @@ else
 	fail(tostring(val))
 end
 
--- 73
+-- 79
 announce("Calling toutf8 with bogus encoding, should fail")
 ok = pcall(ltcl.toutf8, tcl, 'x', 'bogusencoding')
 check(not ok)
 
--- 74
+-- 80
 announce("Calling fromutf8 with bogus encoding, should fail")
 ok = pcall(ltcl.fromutf8, tcl, 'x', 'bogusencoding')
 check(not ok)
 
 ----- vals tests -----
 
--- 75
+-- 81
 announce("Calling list using callt with args {x=tcl:vals(1)}, should return a table {1}")
 ok, val = pcall(ltcl.callt, tcl, 'list', {tcl:vals(1)})
 if ok then
@@ -712,7 +759,7 @@ else
 	fail(tostring(val))
 end
 
--- 76
+-- 82
 announce("Calling list using callt with args {tcl:vals(1,2,3)}, should return a table {'-x',1,2,3}")
 ok, val = pcall(ltcl.callt, tcl, 'list', {tcl:vals(1,2,3)})
 if ok then
@@ -721,7 +768,7 @@ else
 	fail(tostring(val))
 end
 
--- 77
+-- 83
 announce("Making argument list from {x=tcl:vals(1,2,3),y=tcl:vals(4,5)} then Tcalling list with this list, should return a table {'-x',1,2,3,'-y',4,5} or {'-y',4,5,'-x',1,2,3}")
 ok, args = pcall(ltcl.makearglist, tcl, {x=tcl:vals(1,2,3),y=tcl:vals(4,5)})
 ok, val = pcall(ltcl.callt, tcl, 'list', args)
@@ -733,7 +780,7 @@ end
 
 ----- checkflags tests -----
 
--- 78
+-- 84
 announce("Setting flags to 1+2+8+32+64+256, checking against 1,2,4,6,32,33, should return 1,2,nil,nil,32,33")
 flags = 1+2+8+32+64+256
 ok, a,b,c,d,e,f = pcall(ltcl.checkflags, tcl, flags, 1,2,4,6,32,33)
@@ -743,19 +790,19 @@ else
 	fail(tostring(a))
 end
 
--- 79
+-- 85
 announce("Checking flags against 'bla', should fail")
 ok = pcall(ltcl.checkflags, tcl, flags, 'bla')
 check(not ok)
 
--- 80
+-- 86
 announce("Checking 'bla' against 1,2,4, should fail")
 ok = pcall(ltcl.checkflags, tcl, 'bla', 1,2,4)
 check(not ok)
 
 ----- tracevar tests -----
 
--- 81
+-- 87
 announce("Setting write trace on tcl var tracea, then setting tracea to 42. Should call trace func with a, nil, and at least ltcl.TRACE_WRITES")
 trace_ok =false
 function tracevara(n1, n2, flags)
@@ -778,7 +825,7 @@ else
 	fail(tostring(msg))
 end
 
--- 82
+-- 88
 announce("Now unsetting said variable, trace should still be there and behave as before")
 ok, msg = pcall(ltcl.unsetvar, tcl, 'tracea')
 collectgarbage()
@@ -794,7 +841,7 @@ else
 	fail(tostring(msg))
 end
 
--- 83
+-- 89
 announce("Setting write trace on tcl var tracea (which is 42), that changes tracea to 17 before reading. Should call trace func with a, nil, and at least ltcl.TRACE_READS, and getvar should return 17")
 trace_ok = false
 function tracevara2(n1, n2, flags)
