@@ -7,12 +7,13 @@
  -
  - Gunnar Zötl <gz@tset.de>, 2010.
  - Released under MIT/X11 license. See file LICENSE for details.
+ -
+ - 2011-10-17 adjusted for ltk version 2
 --]]
 
-require "ltk"
+ltk = require "ltk"
 
-img = ltk.image{'create', 'photo', width=256, height=256}
-imgcmd = ltk.wcmd(img)
+img = ltk.image.create{'samples/photo', width=256, height=256}
 
 function drawimg()
 	local drawfn
@@ -23,7 +24,7 @@ function drawimg()
 			for x=0, 255 do
 				for y=0, 255 do
 					col = string.format("#%02X%02X%02X", x, y, z)
-					imgcmd{'put', col, to = ltk.vals(x, y)}
+					img:set(x, y, col)
 					if y%63 == 0 then
 						ltk.after{'idle', drawfn}
 						coroutine.yield()
@@ -33,17 +34,16 @@ function drawimg()
 			z = (z + 51) % 256
 		end
 	end)
-	ltk.after{'idle', drawfn}
+	ltk.after.idle(drawfn)
 end
 
 c=ltk.canvas { width=256, height=256 }
-ccmd = ltk.wcmd(c)
-ccmd{'create', 'image', 0, 0, image=img, anchor='nw'}
+c:create_image{0, 0, image=img, anchor='nw'}
 
 b = ltk.button {text="Close", command=function() ltk.exit() end}
 
-ltk.grid{c, row=1}
-ltk.grid{b, row=2}
+c:grid{row=1}
+b:grid{row=2}
 -- show window and all
 ltk.update()
 
