@@ -3,8 +3,10 @@
 --[[
  - tkhello.lua
  -
- - Gunnar Zötl <gz@tset.de>, 2010.
+ - Gunnar Zötl <gz@tset.de>, 2010, 2011.
  - Released under MIT/X11 license. See file LICENSE for details.
+ -
+ - 2011-10-12 adjusted for ltk version 2
 --]]
 
 -- straight port of Tk8.5 widget example to ltk, original header follows:
@@ -15,7 +17,7 @@
 --#
 --# RCS: @(#) $Id: hello,v 1.4 2003/09/30 14:54:30 dkf Exp $
 
-require "ltk"
+ltk = require "ltk"
 
 -- The first line below creates the button, and the second line
 -- asks the packer to shrink-wrap the application's main window
@@ -25,6 +27,6 @@ hello = ltk.button{text="Hello, world", command=function()
 		print "Hello, world!"
 		ltk.exit()
 	end}
-ltk.pack{hello}
+hello:pack()
 
 ltk.mainloop()
